#!/usr/bin/python

"""
windowed_filter(array, winfunc, s)
with winfuncs :
  windowed_mean_point
  windowed_median_point

Command line bindings for filtering 2 column ACSII data.
"""

import numpy

def windowed_mean_point(array, i, width=50) :
    """
    Filter data with a windowed average (mean).
    The ith point is replaced by the average of the points with indices in
    the range i +/- width (inclusive).

    This function returns the ith point in the filtered array,
    given the raw array and window half-width.

    The weights on the average are uniform at the moment.
    """
    x=0
    n=0
    for j in range(i-width, i+width) :
        if j >= 0 and j < len(array) :
            x+=array[j]
            n+=1
    x /= float(n)
    return x

def windowed_median_point(array, i, width=9) :
    """
    Filter data with a windowed median.
    The ith point is replaced by the median of the points with indices in
    the range i +/- width (inclusive).

    This function returns the ith point in the filtered array,
    given the raw array and window half-width.
    """
    imin = i-width
    if imin < 0 : imin = 0
    imax = i+width
    if imax >= len(array) : imax = len(array-1)
    slice = array[imin:imax+1].copy()
    slice.sort()
    imid = numpy.floor((imax-imin)/2)
    return slice[imid]

def windowed_filter(array, winfunc, width=None) :
    """
    Filter data with a windowing function winfunc.
    The ith point is replaced by the winfunc(array, i, s).

    See the windowed_* functions for possible winfunc options.
    """
    out = array.copy()
    if width == None :
        win = lambda i : winfunc(array, i) # user winfunc's default s
    else :
        win = lambda i : winfunc(array, i, width)
    for i in range(len(out)) :
        out[i] = win(i)
    return out    


# commandline interface functions
import scipy.io, sys

def read_data(ifile):
    """
    ifile can be a filename string or open (seekable) file object.
    returns (column 1 array, column 2 array)
    """
    if ifile == None :  ifile = sys.stdin
    data=scipy.io.read_array(ifile)
    return (data[:,0], data[:,1])

def write_data(ofile, x, y):
    """
    ofile can be a filename string or open (seekable) file object.
    """
    data = numpy.zeros((len(x),2))
    data[:,0] = x
    data[:,1] = y
    if ofile == None :  ofile = sys.stdout
    scipy.io.write_array(ofile, data, separator='\t')

if __name__ == '__main__' :
    # command line interface
    from optparse import OptionParser
    
    usage_string = ('%prog <input-file>\n'
                    '2008, W. Trevor King.\n'
                    '\n'
                    'Apply various filters to data y(x).'
                    '<input-file> should be whitespace-delimited, 2 column ASCII\n'
                    'without a header line.\n'
                    'e.g: "<x>\\t<y>\\n"')
    parser = OptionParser(usage=usage_string, version='%prog 0.1')
    parser.add_option('-o', '--output-file', dest='ofilename',
                      help='write output to FILE (default stdout)',
                      type='string', metavar='FILE')
    parser.add_option('-w', '--width', dest='width', default=None,
                      help='window width (i +/- width, inclusive)',
                      type='int', metavar='WIDTH')
    parser.add_option('-t', '--type', dest='type', default='mean',
                      help='filter type (default %default)',
                      type='string', metavar='TYPE')
    parser.add_option('-v', '--verbose', dest='verbose', action='store_true',
                      help='Print lots of debugging information',
                      default=False)

    options,args = parser.parse_args()
    parser.destroy()
    assert len(args) >= 1, "Need an input file"
        
    ifilename = args[0]

    x,y = read_data(ifilename)

    if options.type == 'mean' :
        winfunc = windowed_mean_point
    elif options.type == 'median' :
        winfunc = windowed_median_point
    else :
        raise Exception, "unrecognized window type '%s'" % options.type

    y = windowed_filter(y, winfunc, width=options.width)

    write_data(options.ofilename, x, y)
